*******************************
define class mtsbase as container
*******************************
	* New custom properties
	cXML = 	''
	cTier = ''
	cAction = ''
	cAppStartPath = ''

	******************
	procedure error
	******************
		lparameters tnError, tcMethod, tnLine
		local loXMLDOM, loErrors, loTier, loClass, loAction, i, loElemError, laError, lcDescription

		* create the XMLDOM object
		loXMLDOM = createobject('Microsoft.XMLDOM')
		* have LoadXML wait until it gets the whole string
		loXMLDOM.async = .f.
		* load whatever is already in the cXML property
		if not empty(this.cXML)
			loXMLDOM.LoadXML(this.cXML)
		endif

		loClass = this.GetClass(loXMLDOM, this.cTier, this.class)
		loAction = this.GetXMLElement(loClass, this.cAction)

		*** add the current error under the action
		i = loAction.ChildNodes.length
		loElemError = loXMLDOM.CreateElement('Error' + alltrim(str(i)))
		loAction.AppendChild(loElemError)
		loElemError.SetAttribute('DateTime', datetime())
		*** add all properties of the error
		this.SetErrorSubElement(loXMLDOM, loElemError, 'Number', tnError)

		* get the description of the error
		dimension laError[1]
		aerror(laError)
		* OLE errors are in array element 3, others are in elemet 2
		lcDescription = laError[iif(inlist(tnError, 1427, 1429), 3, 2)]
		this.SetErrorSubElement(loXMLDOM, loElemError, 'Description', lcDescription)
		this.SetErrorSubElement(loXMLDOM, loElemError, 'CallingContext', this.BuildCallingContext())
		this.SetErrorSubElement(loXMLDOM, loElemError, 'Line', tnLine)

		this.cXML = loXMLDOM.xml
	******************
	endproc  && error
	******************


	******************
	procedure BuildCallingContext
	******************
		local lnCounter, lcCallingContext, lcMethodName

		lcCallingContext = ''
		* 0 and 1 give the master program
		lnCounter = 1
		lcMethodName = program(lnCounter)
		* remove the object name, and leave just the method name
		do while NOT Empty(lcMethodName)
			* we really don't need to see the interals of the error handler
			if right(lcMethodName, 5) = 'ERROR'
				exit
			endif
			lcMethodName = substr(lcMethodName, at('.', lcMethodName) + 1)
			* use backslash to make it look like a path
			lcCallingContext = addbs(lcCallingContext) + lcMethodName
			lnCounter = lnCounter + 1
			lcMethodName = program(lnCounter)
		enddo

		return lcCallingContext
	******************
	endproc  && BuildCallingContext
	******************


	******************
	procedure GetXMLElement
	******************
		lparameters toParent, tcName
		local loXMLDOM, loCollection, loElement

		loCollection = toParent.GetElementsByTagName(tcName)
		* if not found, length will be 0
		if loCollection.length = 0
			loXMLDOM = ;
			  iif(vartype(toParent.ownerDocument)=='O', toParent.ownerDocument, toParent)
			* make the element if it's not there yet
			loElement = loXMLDOM.CreateElement(tcName)
			toParent.AppendChild(loElement)
		else
			* otherwise return the existing one
			loElement = loCollection.item(loCollection.length - 1)
		endif

		return loElement
	******************
	endproc  && GetXMLElement
	******************

	******************
	procedure ConcatenateXMLs
	******************
		lparameters tcSrc, tcDest
		local loXMLDOMOuter, loElemErrors, loXMLDOMInner, loInnerTier 
		local loInnerClass, loOuterClass, indexTiers, indexClasses

		* load the xml's
		loXMLDOMOuter = createobject('Microsoft.XMLDOM')
		loXMLDOMOuter.async = .f.
		if not empty(tcDest)
			loXMLDOMOuter.LoadXML(tcDest)
		endif

		loXMLDOMInner = createobject('Microsoft.XMLDOM')
		loXMLDOMInner.async = .f.
		if not empty(tcSrc)
			loXMLDOMInner.LoadXML(tcSrc)
		endif

		* get the highest level Error element
		loElemErrors = this.GetXMLElement(loXMLDOMInner, 'Errors')
		* do all the tiers that appear
		for indexTiers=0 to loElemErrors.ChildNodes.length-1
			loInnerTier = loElemErrors.ChildNodes.item(indexTiers)
			* then do all the Classes that appear
			for indexClasses=0 to loInnerTier.ChildNodes.length-1
				loInnerClass = loInnerTier.ChildNodes.item(indexClasses)
				* and copy the outer class branch into the inner class element
				loOuterClass = this.GetClass(loXMLDOMOuter, loInnerTier.NodeName, loInnerClass.NodeName)
				this.CopyXMLBranch(loInnerClass, loOuterClass)
			endfor  && each loInnerTier 
		endfor  && each loInnerTier 

		* return the resulting combined xml's
		this.cXML = loXMLDOMOuter.xml
	******************
	endproc  && ConcatenateXMLs
	******************

	******************
	procedure GetClass
	******************
		lparameters toDOM, tcTier, tcClass
		LOCAL loErrors, loTier, loClass

		loErrors = this.GetXMLElement(toDOM, 'Errors')
		loTier = this.GetXMLElement(loErrors, tcTier)
		loClass = this.GetXMLElement(loTier, tcClass)

		return loClass
	******************
	endproc  && GetClass
	******************

	******************
	procedure CopyXMLBranch
	******************
		lparameters toSrcClass, toDestClass
		LOCAL loSrcAction, loDestAction, loSrcError, loDestError, indexActions, indexErrors

		* do each action in the source
		for indexActions=0 to toSrcClass.ChildNodes.length-1 
			* look for the same action in the destination
			loSrcAction = toSrcClass.ChildNodes.item(indexActions)
			loDestAction = this.GetXMLElement(toDestClass, loSrcAction.NodeName)
			* then copy all the errors under that action
			for indexErrors=0 to loSrcAction.ChildNodes.length-1
				loSrcError = loSrcAction.ChildNodes.item(indexErrors)
				loDestError = loSrcError.CloneNode(.t.)
				loDestAction.AppendChild(loDestError)
			endfor  && each loSrcAction 
		endfor  && each loSrcError 
	******************
	endproc  && CopyXMLBranch
	******************

	******************
	procedure LogErrors
	******************
		LOCAL loSharedGroupManager, lnIsolationMode, lnReleaseMode, loSharedGroup, llExists
		LOCAL loSrc, loDest, loDOMSrc, loDOMDest, loSharedErrorLogFile, lcFileContents, lcErrorFileName, loMTX
		LOCAL loContext

		* use the SPM to create a write "lock" on the error log file
		loMTX = createobject('MtxAS.AppServer.1')
		loContext = loMTX.GetObjectContext()
		if vartype(loContext) = 'O'
			loSharedGroupManager = loContext.CreateInstance("MTxSpm.SharedPropertyGroupManager.1")
		endif

		if vartype(loSharedGroupManager) = 'O'
			lnIsolationMode = 1 &&LockMethod
			lnReleaseMode = 0 &&Standard
			* lock with the Tier name, since that's the log file we need
			loSharedGroup = loSharedGroupManager.CreatePropertyGroup(this.cTier, lnIsolationMode, ;
				lnReleaseMode, @llExists)
			loSharedErrorLogFile = loSharedGroup.CreateProperty(this.class, @llExists)
		endif

		lcErrorFileName = home() + this.cTier + '.xml'
		lcFileContents = iif(file(lcErrorFileName), filetostr(lcErrorFileName), '')
		loDOMSrc = createobject('Microsoft.XMLDOM')
		loDOMSrc.async = .f.
		if not empty(this.cXML)
			loDOMSrc.LoadXML(this.cXML)
		endif
		
		loDOMDest= createobject('Microsoft.XMLDOM')
		loDOMDest.async = .f.
		if file(lcErrorFileName)
			loDOMDest.load(lcErrorFileName)
		endif

		if not empty(this.cXML) or not empty(lcFileContents)
			loSrc=this.GetClass(loDOMSrc, this.cTier, this.Class)
			loDest=this.GetClass(loDOMDest, this.cTier, this.Class)
			this.CopyXMLBranch(loSrc, loDest)
		endif
		* write to file the resulting XML string
		set safety off
		strtofile(loDOMDest.XML, home() + this.cTier + '.xml')
	******************
	endproc  && LogErrors
	******************

	******************
	procedure SetErrorSubElement
	******************
		lparameters toXMLDOM, toErrorElem, tcSubElemName, tuSubElemValue
		local loSubElem

		loSubElem = toXMLDOM.CreateElement(tcSubElemName)
		toErrorElem.AppendChild(loSubElem)
		loSubElem.SetAttribute('value', tuSubElemValue)

	******************
	endproc  && GetClass
	******************
	
	******************
	procedure Init
	******************
		if not dodefault()
			return .f.
		endif
		** Thanks to Rick Strahl of West-Wind (www.west-wind.com) for
		** these COM tips...
		SET RESOURCE OFF
		SET EXCLUSIVE OFF
		SET CPDIALOG OFF
		SET DELETED ON
		SET EXACT OFF
		SET SAFETY OFF
		SET REPROCESS TO 2 SECONDS
		 
		*** Force server into unattended mode  any dialog
		*** will cause an error with error message
		sys(2335,0)
		
		if inlist(_vfp.startmode, 2, 3, 5)
			this.cAppStartPath = JustPath(Application.Servername)
			set path to (THIS.cAppStartPath)
		endif
	******************
	endproc  && Init
	******************

 
*******************************
enddefine  && class mstbase
*******************************
